-- SHfileOperation.e - Wrap API SHFileOperation and build parameter SHFILEOPSTRUCT
--		SHFileOperation has only been tested with FO_DELETE and one or two files.
--		It has not been tested with the other functions or filenames with wildcards.
--		Arthur R Crump, 2010

include std/dll.e
include std/sequence.e
include std/machine.e
include std/filesys.e

constant user32 = open_dll("user32.dll"),
		shell32 = open_dll("shell32.dll"),
		xGetActiveWindow = define_c_func(user32,"GetActiveWindow",{},C_HANDLE),
		xSHFileOp = define_c_func(shell32,"SHFileOperationA",{C_POINTER},C_UINT)
public constant
		FO_DELETE = #03,		-- The delete function; the only one tested
		-- The following flags affect the delete operation (as well as other functions)
		FOF_SILENT = #0004,				-- Do not create a progress report box (default)
		FOF_NOCONFIRMATION = #0010,		-- Don't prompt the user for confirmation
		FOF_ALLOWUNDO = #0040,			-- !! Required for putting in the recycle bin !!
		FOF_FILESONLY = #0080,			-- Only files, not directories - Not tested
		FOF_SIMPLEPROGESS = #0100,		-- Don't show file names in progress - Not tested
		FOF_NOERRORUI = #0400,			-- Don't create an error dialog boxNot - Not tested
		FOF_NORECURSION = #1000,			-- Don't recurse into folders - Set in all tests
		FOF_NO_CONNECTED_ELEMENTS = #2000,	-- Don't process connected files - Not tested
		--		For example, an HTML file may have connected files in a subfolder
		FOF_WANTNUKEWARNING = #4000, 	-- During delete operation, warn if the file(s)
		-- cannot be moved to the recycle bin (partially overrides FOF_NOCONFIRMATION)
		--
		-- The following functions have not been tested. only FO_DELETE has been tried
		FO_MOVE = #01,
		FO_COPY = #02,
		FO_RENAME = #04,
		--  The following constants do not affect the delete operation but have been
		--	included in case anyone wishes to try out the move, copy and rename functions
		FOF_MULTDESTFILES = #0001,			-- New name per file instead of one folder
--		FOF_CONFIRMMOUSE = #0002,			---- Not currently used by Microsoft ----
		FOF_RENAMECOLLISION = #0008,		-- If new file already exists, use a new name
		FOF_WANTMAPPINGHANDLE = #0020,
		-- The use of FOF_WANTMAPPINGHANDLE would require additional programming
		FOF_NOCONFIRMMKDIR = #0200,			-- Create a new directory silently if needed
		FOF_NOCOPYSECURITYATTRIBS = #0800

--	allocate_file_list returns a pointer to the absolute paths of the supplied files,
--	NULL separated and double NULL terminated.
function allocate_file_list(sequence files, integer cleanup = 0)
	if length(files) = 0 then
		return 0		-- Calling routine should treat this as an error
	elsif atom(files[1]) then
		files = {files}
	end if
	for n = 1 to length(files) do
		files[n] = canonical_path(files[n])
	end for
	files = join(files,0)
	return allocate_string(files&0,cleanup)		-- A list, so double NULL termination
end function

--	Implementation notes:
--	The SHFileOperation API is described in the Microsoft developers site at:
--	http://msdn.microsoft.com/en-us/library/bb759795(v=VS.85).aspx
--	It requires an SHFILEOPSTRUCT structure which is described in:
--	http://msdn.microsoft.com/en-us/library/bb762164(VS.85).aspx
--	The pFrom and pTo fields of SHFILEOPSTRUCT are pointers to single or multiple file
--	names terminated by double-NULLs. Multiple file names are separated by NULLs.
--	allocate_file_names is used to guarantee absolute paths, since relative paths can cause
--	undesirable behaviour, including the inability to put any files in the recycle bin.
--	The return code is 0 if successful or partially so, non-zero on error.

public function SHFileOperation(
			integer wFunc=FO_DELETE,	-- The function that SHFileOperation will perform.
			sequence oldfile,			-- The file or files to be deleted/moved etc.
			object newfile=0,			-- The destination file or directory (Not tested).
			atom flags,					-- Combination of FOF_ flags (see above).
			object ProgTitle=0 )		-- A title for the progress report box, if any.
atom hwnd, pStruct, pFrom, pTo, pTitle
integer sts
	hwnd = c_func(xGetActiveWindow,{})
	pFrom = allocate_file_list(oldfile,1)
	if wFunc = FO_DELETE then
		pTo = 0
		flags = and_bits(flags,#75D4)	-- Restrict the allowed flags
	else	-- Copy, Move or Rename: Not tested.
		pTo = allocate_file_list(newfile&0,1)
	end if
	if sequence(ProgTitle) then
		pTitle = allocate_string(ProgTitle)
	else
		pTitle = 0
--		flags = or_bits(flags,FOF_SILENT)
	end if
	pStruct = allocate(32,1)
	poke4(pStruct, {hwnd,wFunc,pFrom,pTo,flags,0,0,pTitle} )
	sts = c_func(xSHFileOp,{pStruct})
	if sts = 0 and peek4u(pStruct+20) then
		sts = -1
	end if
	return sts
end function

--**
-- Transfers a file or files to the recycle bin.
--
-- Parameters:
-- # ##files## : A sequence; a file name or a sequence of file names
-- # ##FOF_flags## : An optional integer, a combination of FOF_ flags
-- The default flags  are suitable for deleting a single file with confirmation.
--    * FOF_SILENT  Don't create a progress report box (default: set)
--    * FOF_NOCONFIRMATION  Don't prompt the user for confirmation (default: clear)
--    * FOF_ALLOWUNDO  Will always be set by **binFile** for moving to the recycle bin !
--    * FOF_FILESONLY  Only delete files, not directories (default:set)
--    * FOF_SIMPLEPROGRESS  Don't include file names in the progress report box
--    * FOF_NOERRORUI  Don't create an error dialog box (default: clear)
--    * FOF_NORECURSION  Don't recurse into folders (default: set).
--    * FOF_NO_CONNECTED_ELEMENTS  Don't delete connected HTML files (default: clear).
--    * FOF_WANTNUKEWARNING  Warn if file cannot be recycled (default: set).
-- # ##progTitle## : An optional title for the progress report box or 0.
--
-- Returns:
--   An **integer**, 0 if successful, otherwise an error number.
--   the most common error number is 1026 (=#402) which may occur if one or more of the
--   files do not exist, but can also occur with other incorrect parameters.
--
public function binFile(sequence files, atom FOF_flags=#50C4, object progTitle=0)
	return SHFileOperation(FO_DELETE,files,,or_bits(FOF_flags,FOF_ALLOWUNDO),progTitle )
end function
5.0